import sys
import os
import numpy as np
from PyQt5.QtWidgets import (QApplication, QMainWindow, QWidget, QVBoxLayout, 
                             QHBoxLayout, QLabel, QLineEdit, QPushButton, 
                             QComboBox, QTabWidget, QFrame, QFileDialog, QSpinBox)
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QCursor
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import matplotlib.pyplot as plt
from scipy import signal

# Global constants for UI dimensions
OPTION_FRAME_HEIGHT = 50
WINDOW_INITIAL_WIDTH = 1200
WINDOW_INITIAL_HEIGHT = 800
DEBUG = True  # Debug flag

def debug_print(*args):
    if DEBUG:
        print("[DEBUG]", *args)

class SignalProcessingApp(QMainWindow):
    def __init__(self):
        super().__init__()
        self.time_canvas = None
        self.time_fig = None
        self.time_axes = None
        self.spectrum_canvas = None
        self.spectrum_fig = None
        self.spectrum_axes = None
        self.initUI()
    
    def initUI(self):
        # Set window title and size
        self.setWindowTitle('IQ Plotter')
        self.resize(WINDOW_INITIAL_WIDTH, WINDOW_INITIAL_HEIGHT)
        
        # Create central widget and main layout
        central_widget = QWidget()
        main_layout = QVBoxLayout()
        central_widget.setLayout(main_layout)
        self.setCentralWidget(central_widget)
        
        # File selection section
        file_layout = QHBoxLayout()
        file_label = QLabel('File:')
        self.file_input = QLineEdit()
        file_button = QPushButton('open')
        file_button.clicked.connect(self.on_file_open)
        
        file_layout.addWidget(file_label)
        file_layout.addWidget(self.file_input)
        file_layout.addWidget(file_button)
        main_layout.addLayout(file_layout)
        
        # Common info section
        common_info_layout = QHBoxLayout()
        common_info_layout.setSpacing(5)  # Reduce spacing between groups to 5 pixels
        
        # Create a container widget to hold all groups
        common_info_container = QWidget()
        common_info_container_layout = QHBoxLayout()
        common_info_container_layout.setSpacing(5)  # Minimal spacing between groups
        common_info_container_layout.setContentsMargins(0, 0, 0, 0)  # Remove margins
        common_info_container.setLayout(common_info_container_layout)
        
        # Channel BW pair
        channel_bw_widget = QWidget()
        channel_bw_widget.setFixedWidth(180)  # Slightly reduced width
        channel_bw_group = QHBoxLayout()
        channel_bw_group.setSpacing(2)  # Minimal spacing between label and control
        channel_bw_label = QLabel('Channel BW(Mhz):')
        self.channel_bw_input = QLineEdit('20')
        self.channel_bw_input.setFixedWidth(60)
        channel_bw_group.addWidget(channel_bw_label)
        channel_bw_group.addWidget(self.channel_bw_input)
        channel_bw_group.addStretch()
        channel_bw_widget.setLayout(channel_bw_group)
        
        # Add spacing widget between groups
        spacing_widget = QWidget()
        spacing_widget.setFixedWidth(20)  # Add 20 pixels of space
        
        # Sample Rate pair
        sample_rate_widget = QWidget()
        sample_rate_widget.setFixedWidth(180)  # Slightly reduced width
        sample_rate_group = QHBoxLayout()
        sample_rate_group.setSpacing(2)
        sample_rate_label = QLabel('Sample Rate (M sps):')
        self.sample_rate_input = QLineEdit('30.72')
        self.sample_rate_input.setFixedWidth(60)
        sample_rate_group.addWidget(sample_rate_label)
        sample_rate_group.addWidget(self.sample_rate_input)
        sample_rate_group.addStretch()
        sample_rate_widget.setLayout(sample_rate_group)
        
        # FFT bin size pair
        fft_bin_group = QHBoxLayout()
        fft_bin_group.setSpacing(2)
        fft_bin_label = QLabel('FFT bin size')
        self.fft_bin_combo = QComboBox()
        self.fft_bin_combo.addItem('2048')
        self.fft_bin_combo.addItem('1024')
        fft_bin_group.addWidget(fft_bin_label)
        fft_bin_group.addWidget(self.fft_bin_combo)
        
        # Start percentage pair
        start_group = QHBoxLayout()
        start_group.setSpacing(3)
        start_label = QLabel('Start(%):')
        self.start_spin = QSpinBox()
        self.start_spin.setRange(0, 95)
        self.start_spin.setSingleStep(5)
        self.start_spin.setValue(0)
        self.start_spin.setFixedWidth(80)  # Set fixed width for start spin box
        self.start_spin.valueChanged.connect(self.on_start_changed)
        start_group.addWidget(start_label)
        start_group.addWidget(self.start_spin)
        
        # End percentage pair
        end_group = QHBoxLayout()
        end_group.setSpacing(3)
        end_label = QLabel('End(%):')
        self.end_spin = QSpinBox()
        self.end_spin.setRange(5, 100)
        self.end_spin.setSingleStep(5)
        self.end_spin.setValue(100)
        self.end_spin.setFixedWidth(80)  # Set fixed width for end spin box
        self.end_spin.valueChanged.connect(self.on_end_changed)
        end_group.addWidget(end_label)
        end_group.addWidget(self.end_spin)
        
        # File size label pair
        filesize_widget = QWidget()
        filesize_widget.setFixedWidth(250)  # Fixed width for consistent layout
        filesize_group = QHBoxLayout()
        filesize_group.setSpacing(2)
        filesize_label = QLabel('File size in ms(samples):')
        self.filesize_value = QLabel('0')  # Initialize with 0
        filesize_group.addWidget(filesize_label)
        filesize_group.addWidget(self.filesize_value)
        filesize_group.addStretch()
        filesize_widget.setLayout(filesize_group)
        
        # Add all groups to container layout
        common_info_container_layout.addWidget(channel_bw_widget)
        common_info_container_layout.addWidget(spacing_widget)  # Add spacing widget
        common_info_container_layout.addWidget(sample_rate_widget)
        common_info_container_layout.addLayout(fft_bin_group)
        common_info_container_layout.addLayout(start_group)
        common_info_container_layout.addLayout(end_group)
        common_info_container_layout.addWidget(filesize_widget)  # Add filesize widget
        common_info_container_layout.addStretch()  # Add stretch at the end to keep groups left-aligned
        
        # Add container to main layout
        common_info_layout.addWidget(common_info_container)
        main_layout.addLayout(common_info_layout)
        
        # Plot tabs section
        self.plot_tabs = QTabWidget()
        time_tab = QWidget()
        spectrum_tab = QWidget()
        
        self.plot_tabs.addTab(time_tab, 'Time')
        self.plot_tabs.addTab(spectrum_tab, 'Spectrum')
        
        # Time plot layout
        time_layout = QVBoxLayout()
        
        # Time Plot Option Frame
        time_plot_option_frame = QFrame()
        time_plot_option_frame.setFrameShape(QFrame.StyledPanel)
        time_plot_option_frame.setFixedHeight(OPTION_FRAME_HEIGHT)
        time_plot_option_layout = QHBoxLayout()
        
        # Add Value Format and Y Scale combo boxes
        value_format_label = QLabel('Value Format:')
        self.time_value_format_combo = QComboBox()
        self.time_value_format_combo.addItems(['Raw', 'Normalize'])
        
        y_scale_label = QLabel('Y Scale:')
        self.time_y_scale_combo = QComboBox()
        self.time_y_scale_combo.addItems(['Linear', 'dB'])
        
        time_plot_option_layout.addWidget(value_format_label)
        time_plot_option_layout.addWidget(self.time_value_format_combo)
        time_plot_option_layout.addWidget(y_scale_label)
        time_plot_option_layout.addWidget(self.time_y_scale_combo)
        
        # Add measurement tools
        measurement_label = QLabel('Measurements:')
        self.measurement_combo = QComboBox()
        self.measurement_combo.addItems(['None', 'Peak', 'RMS', 'Statistics'])
        
        time_plot_option_layout.addWidget(measurement_label)
        time_plot_option_layout.addWidget(self.measurement_combo)
        
        # Add spacer to push Plot button to the right
        time_plot_option_layout.addStretch()
        
        # Add Plot button for time plot
        self.btnPlotTime = QPushButton('Plot')
        debug_print("Creating Time Plot button")
        self.btnPlotTime.clicked.connect(lambda: self.on_time_plot_clicked())
        debug_print("Connected Time Plot button click handler")
        time_plot_option_layout.addWidget(self.btnPlotTime)
        
        time_plot_option_frame.setLayout(time_plot_option_layout)
        time_layout.addWidget(time_plot_option_frame)
        
        # Time Plot Frame
        time_plot_frame = QFrame()
        time_plot_frame.setFrameShape(QFrame.StyledPanel)
        
        # Create layout for time plot frame
        time_plot_layout = QVBoxLayout()
        time_plot_frame.setLayout(time_plot_layout)
        time_plot_frame.setObjectName("time_plot_frame")  # Add object name for reference
        
        time_layout.addWidget(time_plot_frame)
        
        time_tab.setLayout(time_layout)
        
        # Spectrum plot layout
        spectrum_layout = QVBoxLayout()
        
        # Spectrum Plot Option Frame
        spectrum_plot_option_frame = QFrame()
        spectrum_plot_option_frame.setFrameShape(QFrame.StyledPanel)
        spectrum_plot_option_frame.setFixedHeight(OPTION_FRAME_HEIGHT)
        spectrum_plot_option_layout = QHBoxLayout()
        
        # Add Value Format and Y Scale combo boxes
        value_format_label = QLabel('Value Format:')
        self.value_format_combo = QComboBox()
        self.value_format_combo.addItems(['Normalize', 'Raw'])
        self.value_format_combo.currentTextChanged.connect(self.on_plot_options_changed)
        
        y_scale_label = QLabel('Y Scale:')
        self.y_scale_combo = QComboBox()
        self.y_scale_combo.addItems(['dB', 'Linear'])
        self.y_scale_combo.currentTextChanged.connect(self.on_plot_options_changed)
        
        spectrum_plot_option_layout.addWidget(value_format_label)
        spectrum_plot_option_layout.addWidget(self.value_format_combo)
        spectrum_plot_option_layout.addWidget(y_scale_label)
        spectrum_plot_option_layout.addWidget(self.y_scale_combo)
        
        # Add Power Range controls
        power_min_label = QLabel('Power Min:')
        self.power_min_input = QLineEdit('-80')
        power_max_label = QLabel('Power Max:')
        self.power_max_input = QLineEdit('0')
        
        spectrum_plot_option_layout.addWidget(power_min_label)
        spectrum_plot_option_layout.addWidget(self.power_min_input)
        spectrum_plot_option_layout.addWidget(power_max_label)
        spectrum_plot_option_layout.addWidget(self.power_max_input)
        
        # Add spacer to push Plot button to the right
        spectrum_plot_option_layout.addStretch()
        
        # Add Plot button for spectrum plot
        self.btnPlotSpectrum = QPushButton('Plot')
        debug_print("Creating Spectrum Plot button")
        self.btnPlotSpectrum.clicked.connect(lambda: self.on_spectrum_plot_clicked())
        debug_print("Connected Spectrum Plot button click handler")
        spectrum_plot_option_layout.addWidget(self.btnPlotSpectrum)
        
        spectrum_plot_option_frame.setLayout(spectrum_plot_option_layout)
        spectrum_layout.addWidget(spectrum_plot_option_frame)
        
        # Spectrum Plot Frame
        spectrum_plot_frame = QFrame()
        spectrum_plot_frame.setFrameShape(QFrame.StyledPanel)
        spectrum_plot_layout = QVBoxLayout()
        spectrum_plot_frame.setLayout(spectrum_plot_layout)
        spectrum_plot_frame.setObjectName("spectrum_plot_frame")
        
        spectrum_layout.addWidget(spectrum_plot_frame)
        
        spectrum_tab.setLayout(spectrum_layout)
        
        main_layout.addWidget(self.plot_tabs)
        
    def on_file_open(self):
        # Open file dialog in the user's home directory
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            "Select IQ Data File",
            "",  # Start in default/home directory
            "All Files (*);;Binary Files (*.bin);;DAT Files (*.dat)"
        )
        
        # If a file was selected (dialog not cancelled), update the text box
        if file_path:
            self.file_input.setText(file_path)
            try:
                # Calculate file size in samples and milliseconds
                file_size = os.path.getsize(file_path)
                num_samples = file_size // (4 * 2)  # 4 bytes per float32, 2 for I/Q
                sample_rate = float(self.sample_rate_input.text()) * 1e6  # Convert to Hz
                duration_ms = (num_samples / sample_rate) * 1000  # Convert to milliseconds
                
                # Update the file size label with both milliseconds and samples
                self.filesize_value.setText(f"{duration_ms:.2f} ({num_samples:,})")
                debug_print(f"File size: {file_size} bytes, {num_samples:,} samples, {duration_ms:.2f} ms")
            except Exception as e:
                debug_print(f"Error calculating file size: {str(e)}")
                self.filesize_value.setText("Error")
    
    def process_signal(self):
        # Main signal processing method
        # Implement signal processing logic here
        pass
    
    def update_time_plot(self):
        # Update time domain plot
        pass
    
    def update_spectrum_plot(self):
        # Update frequency spectrum plot
        pass
    
    def on_start_changed(self, value):
        """Ensure start value is always less than end value"""
        if value >= self.end_spin.value():
            self.start_spin.setValue(self.end_spin.value() - 5)
    
    def on_end_changed(self, value):
        """Ensure end value is always greater than start value"""
        if value <= self.start_spin.value():
            self.end_spin.setValue(self.start_spin.value() + 5)

    def get_data_slice(self, data):
        """Get the specified percentage range of data"""
        start_idx = int(len(data) * self.start_spin.value() / 100)
        end_idx = int(len(data) * self.end_spin.value() / 100)
        return data[start_idx:end_idx]
    
    def on_time_plot_clicked(self):
        debug_print("Time Plot button clicked")
        file_path = self.file_input.text()
        if not file_path:
            debug_print("No file path provided")
            return
            
        if not os.path.exists(file_path):
            debug_print(f"File does not exist: {file_path}")
            return
            
        try:
            # Change cursor to wait cursor
            QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
            
            # Read IQ data from file
            iq_data = np.fromfile(file_path, dtype=np.float32)
            # Get the specified range
            iq_data = self.get_data_slice(iq_data)
            debug_print(f"Using data range: {self.start_spin.value()}% to {self.end_spin.value()}%")
            debug_print(f"Data size after slicing: {len(iq_data)}")
            
            # Reshape into I and Q components
            iq_data = iq_data.reshape(-1, 2)
            i_data = iq_data[:, 0]
            q_data = iq_data[:, 1]
            
            # Calculate magnitude
            magnitude = np.abs(i_data + 1j * q_data)
            
            # Get plot options
            value_format = self.time_value_format_combo.currentText()
            y_scale = self.time_y_scale_combo.currentText()
            measurement = self.measurement_combo.currentText()
            
            # Process data based on value format and y scale
            if value_format == 'Normalize':
                # Normalize all signals to [0,1] range
                i_max = np.max(np.abs(i_data))
                q_max = np.max(np.abs(q_data))
                mag_max = np.max(magnitude)
                
                i_data = i_data / i_max
                q_data = q_data / q_max
                magnitude = magnitude / mag_max
            
            # Convert to dB if selected
            if y_scale == 'dB':
                # Avoid log10(0) by adding small value and masking zeros
                i_data_safe = np.where(np.abs(i_data) > 0, np.abs(i_data), np.finfo(float).tiny)
                q_data_safe = np.where(np.abs(q_data) > 0, np.abs(q_data), np.finfo(float).tiny)
                magnitude_safe = np.where(magnitude > 0, magnitude, np.finfo(float).tiny)
                
                i_data_db = 20 * np.log10(i_data_safe)
                q_data_db = 20 * np.log10(q_data_safe)
                magnitude_db = 20 * np.log10(magnitude_safe)
                
                i_data = i_data_db
                q_data = q_data_db
                magnitude = magnitude_db
            
            # Create time axis
            time_axis = np.arange(len(i_data)) / float(self.sample_rate_input.text())
            
            # Create new figure and subplots
            if self.time_fig is not None:
                plot_frame = self.findChild(QFrame, "time_plot_frame")
                if plot_frame:
                    layout = plot_frame.layout()
                    while layout.count():
                        item = layout.takeAt(0)
                        widget = item.widget()
                        if widget:
                            widget.deleteLater()
            
            self.time_fig = Figure(figsize=(8, 12))
            self.time_canvas = FigureCanvas(self.time_fig)
            self.time_axes = []
            
            # Create subplots with minimal spacing
            for i in range(3):  # Reduced to 3 plots since magnitude dB is now optional
                self.time_axes.append(self.time_fig.add_subplot(3, 1, i+1))
            
            # Set font sizes for all plots
            title_size = 8
            label_size = 7
            tick_size = 6
            
            # Plot I component
            self.time_axes[0].plot(time_axis, i_data)
            self.time_axes[0].set_title('I Component', fontsize=title_size, pad=2)
            self.time_axes[0].set_xlabel('')  # Remove x-axis label
            y_label = 'Amplitude'
            if y_scale == 'dB':
                y_label += ' (dB)'
            if value_format == 'Normalize':
                y_label += ' (Normalized)'
            self.time_axes[0].set_ylabel(y_label, fontsize=label_size)
            
            # Add measurements if selected
            if measurement != 'None':
                if measurement == 'Peak':
                    peak = np.max(np.abs(i_data))
                    self.time_axes[0].axhline(y=peak, color='r', linestyle='--', label=f'Peak: {peak:.2f}')
                elif measurement == 'RMS':
                    rms = np.sqrt(np.mean(i_data**2))
                    self.time_axes[0].axhline(y=rms, color='g', linestyle='--', label=f'RMS: {rms:.2f}')
                elif measurement == 'Statistics':
                    mean = np.mean(i_data)
                    std = np.std(i_data)
                    self.time_axes[0].axhline(y=mean, color='g', linestyle='--', label=f'Mean: {mean:.2f}')
                    self.time_axes[0].axhline(y=mean+std, color='r', linestyle=':', label=f'Std: {std:.2f}')
                    self.time_axes[0].axhline(y=mean-std, color='r', linestyle=':')
                self.time_axes[0].legend(fontsize=tick_size)
            
            self.time_axes[0].tick_params(axis='both', which='major', labelsize=tick_size)
            self.time_axes[0].grid(True)
            
            # Plot Q component with similar modifications
            self.time_axes[1].plot(time_axis, q_data)
            self.time_axes[1].set_title('Q Component', fontsize=title_size, pad=2)
            self.time_axes[1].set_xlabel('')  # Remove x-axis label
            self.time_axes[1].set_ylabel(y_label, fontsize=label_size)
            
            if measurement != 'None':
                if measurement == 'Peak':
                    peak = np.max(np.abs(q_data))
                    self.time_axes[1].axhline(y=peak, color='r', linestyle='--', label=f'Peak: {peak:.2f}')
                elif measurement == 'RMS':
                    rms = np.sqrt(np.mean(q_data**2))
                    self.time_axes[1].axhline(y=rms, color='g', linestyle='--', label=f'RMS: {rms:.2f}')
                elif measurement == 'Statistics':
                    mean = np.mean(q_data)
                    std = np.std(q_data)
                    self.time_axes[1].axhline(y=mean, color='g', linestyle='--', label=f'Mean: {mean:.2f}')
                    self.time_axes[1].axhline(y=mean+std, color='r', linestyle=':', label=f'Std: {std:.2f}')
                    self.time_axes[1].axhline(y=mean-std, color='r', linestyle=':')
                self.time_axes[1].legend(fontsize=tick_size)
            
            self.time_axes[1].tick_params(axis='both', which='major', labelsize=tick_size)
            self.time_axes[1].grid(True)
            
            # Plot magnitude (bottom plot)
            self.time_axes[2].plot(time_axis, magnitude)
            self.time_axes[2].set_title('Magnitude', fontsize=title_size, pad=2)
            self.time_axes[2].set_xlabel('Time (μs)', fontsize=label_size)  # Keep x-axis label only on bottom plot
            self.time_axes[2].set_ylabel(y_label, fontsize=label_size)
            
            if measurement != 'None':
                if measurement == 'Peak':
                    peak = np.max(magnitude)
                    self.time_axes[2].axhline(y=peak, color='r', linestyle='--', label=f'Peak: {peak:.2f}')
                elif measurement == 'RMS':
                    rms = np.sqrt(np.mean(magnitude**2))
                    self.time_axes[2].axhline(y=rms, color='g', linestyle='--', label=f'RMS: {rms:.2f}')
                elif measurement == 'Statistics':
                    mean = np.mean(magnitude)
                    std = np.std(magnitude)
                    self.time_axes[2].axhline(y=mean, color='g', linestyle='--', label=f'Mean: {mean:.2f}')
                    self.time_axes[2].axhline(y=mean+std, color='r', linestyle=':', label=f'Std: {std:.2f}')
                    self.time_axes[2].axhline(y=mean-std, color='r', linestyle=':')
                self.time_axes[2].legend(fontsize=tick_size)
            
            self.time_axes[2].tick_params(axis='both', which='major', labelsize=tick_size)
            self.time_axes[2].grid(True)
            
            # Adjust layout with minimal spacing
            self.time_fig.subplots_adjust(hspace=0.3)
            
            # Add canvas and toolbar to layout
            plot_frame = self.findChild(QFrame, "time_plot_frame")
            if plot_frame:
                layout = plot_frame.layout()
                toolbar = NavigationToolbar(self.time_canvas, plot_frame)
                layout.addWidget(toolbar)
                layout.addWidget(self.time_canvas)
            
            self.time_canvas.draw()
            
        except Exception as e:
            print(f"Error plotting data: {str(e)}")
            if DEBUG:
                import traceback
                traceback.print_exc()
        finally:
            # Restore default cursor
            QApplication.restoreOverrideCursor()
    
    def on_plot_options_changed(self, _):
        """Handle plot options changes and update power range defaults"""
        value_format = self.value_format_combo.currentText()
        y_scale = self.y_scale_combo.currentText()
        debug_print(f"Plot options changed - Value Format: {value_format}, Y Scale: {y_scale}")
        
        if value_format == 'Normalize':
            if y_scale == 'dB':
                self.power_min_input.setText('-80')
                self.power_max_input.setText('0')
            else:  # Linear
                self.power_min_input.setText('0')
                self.power_max_input.setText('1')
        else:  # Raw
            try:
                file_path = self.file_input.text()
                if file_path:
                    iq_data = np.fromfile(file_path, dtype=np.float32)
                    iq_data = iq_data.reshape(-1, 2)
                    complex_data = iq_data[:, 0] + 1j * iq_data[:, 1]
                    if y_scale == 'dB':
                        power = np.abs(complex_data)**2
                        power = 10 * np.log10(np.maximum(power, 1e-10))
                        valid_power = power[np.isfinite(power)]
                        if len(valid_power) > 0:
                            power_min = np.floor(np.min(valid_power))
                            power_max = np.ceil(np.max(valid_power))
                            self.power_min_input.setText(f'{power_min:.0f}')
                            self.power_max_input.setText(f'{power_max:.0f}')
                    else:  # Linear
                        # For linear scale, use magnitude values
                        magnitude = np.abs(complex_data)
                        max_val = np.max(magnitude)
                        min_val = np.max([np.min(magnitude[magnitude > 0]), max_val * 1e-3])
                        
                        self.power_min_input.setText(f'{min_val:.2e}')
                        self.power_max_input.setText(f'{max_val:.2e}')
            except Exception as e:
                debug_print(f"Error calculating raw power range: {str(e)}")
    
    def on_spectrum_plot_clicked(self):
        debug_print("Spectrum Plot button clicked")
        file_path = self.file_input.text()
        if not file_path:
            debug_print("No file path provided")
            return
            
        if not os.path.exists(file_path):
            debug_print(f"File does not exist: {file_path}")
            return
            
        try:
            # Change cursor to wait cursor
            QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
            
            # Read IQ data from file
            iq_data = np.fromfile(file_path, dtype=np.float32)
            # Get the specified range
            iq_data = self.get_data_slice(iq_data)
            debug_print(f"Using data range: {self.start_spin.value()}% to {self.end_spin.value()}%")
            debug_print(f"Data size after slicing: {len(iq_data)}")
            
            # Reshape into I and Q components
            iq_data = iq_data.reshape(-1, 2)
            complex_data = iq_data[:, 0] + 1j * iq_data[:, 1]
            debug_print(f"Complex data shape: {complex_data.shape}")
            
            # Get parameters
            sample_rate = float(self.sample_rate_input.text()) * 1e6  # Convert to Hz
            fft_size = int(self.fft_bin_combo.currentText())
            channel_bw = float(self.channel_bw_input.text()) * 1e6  # Convert to Hz
            debug_print(f"Parameters: sample_rate={sample_rate/1e6}MHz, fft_size={fft_size}, channel_bw={channel_bw/1e6}MHz")
            
            # Get plot options
            value_format = self.value_format_combo.currentText()
            y_scale = self.y_scale_combo.currentText()
            power_min = float(self.power_min_input.text())
            power_max = float(self.power_max_input.text())
            debug_print(f"Plot options - Value Format: {value_format}, Y Scale: {y_scale}")
            debug_print(f"Power range: min={power_min}, max={power_max}")
            
            # Calculate spectrogram
            f, t, Sxx = signal.spectrogram(complex_data, 
                                         fs=sample_rate,
                                         window='hann',
                                         nperseg=fft_size,
                                         noverlap=fft_size//2,
                                         nfft=fft_size,
                                         scaling='spectrum',
                                         mode='complex')
            
            # Calculate spectrum using Welch's method
            f_welch, spectrum = signal.welch(complex_data, 
                                           fs=sample_rate,
                                           window='hann',
                                           nperseg=fft_size,
                                           noverlap=fft_size//2,
                                           nfft=fft_size,
                                           scaling='spectrum',
                                           return_onesided=False)
            
            # Calculate power
            Sxx_power = np.abs(Sxx) ** 2
            spectrum_power = spectrum  # Welch already gives us power
            
            debug_print(f"Initial power ranges:")
            debug_print(f"Sxx_power: min={np.min(Sxx_power):.2e}, max={np.max(Sxx_power):.2e}")
            debug_print(f"spectrum_power: min={np.min(spectrum_power):.2e}, max={np.max(spectrum_power):.2e}")
            
            # Process data based on value format and y scale
            if value_format == 'Normalize':
                # For Normalize mode, always scale to [0,1] range
                Sxx_max = np.max(Sxx_power)
                spectrum_max = np.max(spectrum_power)
                
                # Normalize to [0,1] range using power values directly
                Sxx_norm = Sxx_power / Sxx_max
                spectrum_norm = spectrum_power / spectrum_max
                
                debug_print(f"Normalized ranges:")
                debug_print(f"Sxx_norm: min={np.min(Sxx_norm):.2e}, max={np.max(Sxx_norm):.2e}")
                debug_print(f"spectrum_norm: min={np.min(spectrum_norm):.2e}, max={np.max(spectrum_norm):.2e}")
                
                if y_scale == 'dB':
                    # Convert normalized values to dB
                    Sxx = 10 * np.log10(np.maximum(Sxx_norm, 1e-10))
                    spectrum = 10 * np.log10(np.maximum(spectrum_norm, 1e-10))
                    debug_print(f"After dB conversion:")
                    debug_print(f"Sxx (dB): min={np.min(Sxx):.2f}, max={np.max(Sxx):.2f}")
                    debug_print(f"spectrum (dB): min={np.min(spectrum):.2f}, max={np.max(spectrum):.2f}")
                else:  # Linear
                    # Use normalized power values directly
                    Sxx = Sxx_norm
                    spectrum = spectrum_norm
            else:  # Raw
                if y_scale == 'dB':
                    # Convert power to dB
                    Sxx = 10 * np.log10(np.maximum(Sxx_power, 1e-10))
                    spectrum = 10 * np.log10(np.maximum(spectrum_power, 1e-10))
                    debug_print(f"Raw dB ranges:")
                    debug_print(f"Sxx (dB): min={np.min(Sxx):.2f}, max={np.max(Sxx):.2f}")
                    debug_print(f"spectrum (dB): min={np.min(spectrum):.2f}, max={np.max(spectrum):.2f}")
                else:  # Linear
                    # For Raw Linear mode, use magnitude values
                    Sxx = np.abs(Sxx)  # Convert complex values to magnitude
                    spectrum = np.sqrt(spectrum_power)  # Convert power to magnitude
                    
                    debug_print(f"Raw Linear ranges (using magnitude):")
                    debug_print(f"Sxx: min={np.min(Sxx):.2e}, max={np.max(Sxx):.2e}")
                    debug_print(f"spectrum: min={np.min(spectrum):.2e}, max={np.max(spectrum):.2e}")
                    
                    # Update power range if needed and update local variables accordingly
                    actual_max = max(np.max(Sxx), np.max(spectrum))
                    actual_min = max(np.min(Sxx[Sxx > 0]), np.min(spectrum[spectrum > 0]), actual_max * 1e-3)
                    if abs(float(self.power_max_input.text()) - actual_max) > actual_max * 0.1:
                        self.power_max_input.setText(f'{actual_max:.2e}')
                        self.power_min_input.setText(f'{actual_min:.2e}')
                        power_max = actual_max
                        power_min = actual_min
            
            # Shift frequencies to center
            Sxx = np.fft.fftshift(Sxx, axes=0)
            f = np.fft.fftshift(f)
            spectrum = np.fft.fftshift(spectrum)
            f_welch = np.fft.fftshift(f_welch)
            
            debug_print(f"Final ranges after frequency shift:")
            debug_print(f"Sxx: min={np.min(Sxx):.2e}, max={np.max(Sxx):.2e}")
            debug_print(f"spectrum: min={np.min(spectrum):.2e}, max={np.max(spectrum):.2e}")
            
            # Create edge arrays for pcolormesh
            dt = t[1] - t[0]
            t_edges = np.append(t, t[-1] + dt)
            df = f[1] - f[0]
            f_edges = np.append(f, f[-1] + df)
            
            # Create new figure and subplots
            if self.spectrum_fig is not None:
                plot_frame = self.findChild(QFrame, "spectrum_plot_frame")
                if plot_frame:
                    layout = plot_frame.layout()
                    while layout.count():
                        item = layout.takeAt(0)
                        widget = item.widget()
                        if widget:
                            widget.deleteLater()
            
            self.spectrum_fig = Figure(figsize=(8, 12))
            self.spectrum_canvas = FigureCanvas(self.spectrum_fig)
            
            # Create subplots with 2:1 height ratio
            gs = self.spectrum_fig.add_gridspec(3, 1)
            self.spectrum_axes = []
            self.spectrum_axes.append(self.spectrum_fig.add_subplot(gs[0:2, 0]))  # Top plot takes 2/3
            self.spectrum_axes.append(self.spectrum_fig.add_subplot(gs[2, 0]))    # Bottom plot takes 1/3
            
            # Set font sizes
            title_size = 8
            label_size = 7
            tick_size = 6
            
            # Plot spectrogram
            im = self.spectrum_axes[0].pcolormesh(t_edges * 1e6, f_edges / 1e6, Sxx,
                                               shading='flat',
                                               cmap='viridis')
            
            # Set power limits
            im.set_clim(power_min, power_max)
            
            # Add grid and labels
            self.spectrum_axes[0].set_title('Spectrogram', fontsize=title_size, pad=2)
            self.spectrum_axes[0].set_xlabel('Time (μs)', fontsize=label_size)
            self.spectrum_axes[0].set_ylabel('Frequency (MHz)', fontsize=label_size)
            self.spectrum_axes[0].tick_params(axis='both', which='major', labelsize=tick_size)
            self.spectrum_axes[0].grid(True, alpha=0.3)
            
            # Plot spectrum
            self.spectrum_axes[1].plot(f_welch/1e6, spectrum)
            self.spectrum_axes[1].set_title('Power Spectral Density', fontsize=title_size, pad=2)
            self.spectrum_axes[1].set_xlabel('Frequency (MHz)', fontsize=label_size)
            
            # Set y-axis label based on options
            y_label = 'Power'
            if y_scale == 'dB':
                y_label += ' (dB)'
            if value_format == 'Normalize':
                y_label += ' (Normalized)'
            self.spectrum_axes[1].set_ylabel(y_label, fontsize=label_size)
            
            # Set y-axis limits for spectrum plot
            self.spectrum_axes[1].set_ylim(power_min, power_max)
            
            self.spectrum_axes[1].tick_params(axis='both', which='major', labelsize=tick_size)
            self.spectrum_axes[1].grid(True)
            
            # Set frequency limits based on channel bandwidth
            freq_limit = channel_bw / 2e6  # Convert to MHz
            self.spectrum_axes[0].set_ylim(-freq_limit, freq_limit)
            self.spectrum_axes[1].set_xlim(-freq_limit, freq_limit)
            
            # Adjust layout with minimal spacing
            self.spectrum_fig.subplots_adjust(hspace=0.2)
            
            # Add canvas and toolbar to layout
            plot_frame = self.findChild(QFrame, "spectrum_plot_frame")
            if plot_frame:
                layout = plot_frame.layout()
                toolbar = NavigationToolbar(self.spectrum_canvas, plot_frame)
                layout.addWidget(toolbar)
                layout.addWidget(self.spectrum_canvas)
            
            self.spectrum_canvas.draw()
            debug_print("Plotting completed")
            
        except Exception as e:
            print(f"Error plotting spectrum: {str(e)}")
            if DEBUG:
                import traceback
                traceback.print_exc()
        finally:
            # Restore default cursor
            QApplication.restoreOverrideCursor()

def main():
    app = QApplication(sys.argv)
    main_window = SignalProcessingApp()
    main_window.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()
